import 'dart:developer';
import 'dart:io';

import 'package:flutter/material.dart';
import 'dart:async';

import 'package:cs_ayetstudios_ad/cs_ayetstudios_ad.dart';

void main() {
  runApp(const MyApp());
}

class MyApp extends StatefulWidget {
  const MyApp({Key? key}) : super(key: key);

  @override
  State<MyApp> createState() => _MyAppState();
}

class _MyAppState extends State<MyApp> {
  @override
  void initState() {
    super.initState();
    initSDK();
  }

  // Platform messages are asynchronous, so we initialize in an async method.
  Future<void> initSDK() async {
    try {
      if (Platform.isAndroid) {
        CsAyetstudiosAndroid().initSDK(uid: 'android uid');
        CsAyetstudiosAndroid().setInitilizationFailedListener(initializationFailed);
        CsAyetstudiosAndroid().setUserAvailableBalance(userAvailableBalance);
        CsAyetstudiosAndroid().setUserBalanceChanged(userBalanceChanged);
        CsAyetstudiosAndroid().setUserPendingBalance(userPendingBalance);
      } else if (Platform.isIOS) {
        CsAyetstudiosIos().sdkLogEnable();
        CsAyetstudiosIos().initSDK(appKey: 'ios app key', uid: 'ios uid');
        CsAyetstudiosIos().setUserAvailableBalance(userAvailableBalance);
        CsAyetstudiosIos().setUserBalanceChanged(userBalanceChanged);
        CsAyetstudiosIos().setUserPendingBalance(userPendingBalance);
        CsAyetstudiosIos().userAvailableBalance();
        CsAyetstudiosIos().userPendingBalance();
      }
    } catch (e) {
      log("初始化SDK失败:" + e.toString());
    }
  }

  void initializationFailed(int failed) {
    print('Ayetstudios", "initializationFailed $failed');
  }

  void userAvailableBalance(int balance) {
    print('Ayetstudios available balance = $balance');
  }

  void userBalanceChanged(int balance) {
    print('Ayetstudios balance changed = $balance');
  }

  void userPendingBalance(int balance) {
    print('AyetSdk pending balance = $balance');
  }

  @override
  Widget build(BuildContext context) {
    return MaterialApp(
      home: Scaffold(
        appBar: AppBar(
          title: const Text('Plugin example app'),
        ),
        body: const Center(child: Text('测试初始化Ayetstudios SDK')),
      ),
    );
  }
}
