import 'dart:async';
import 'dart:developer';

import 'package:flutter/services.dart';

typedef UserBalanceChanged = void Function(int balance);
typedef UserAvailableBalance = void Function(int balance);
typedef UserPendingBalance = void Function(int balance);
typedef InitializationFailed = void Function(int failed);
typedef DeductUserBalance = void Function(int success);

class CsAyetstudiosAndroid {
  final MethodChannel _channel = const MethodChannel('cs_ayetstudios_android');
  static final CsAyetstudiosAndroid _instance = CsAyetstudiosAndroid._internal();
  factory CsAyetstudiosAndroid() => _instance;
  CsAyetstudiosAndroid._internal() {
    _channel.setMethodCallHandler(_platformCallHandler);
  }

  late UserBalanceChanged _userBalanceChanged;
  late DeductUserBalance _deductUserBalance;
  late UserAvailableBalance _userAvailableBalance;
  late UserPendingBalance _userPendingBalance;
  late InitializationFailed _initializationFailed;

  Future<void> initSDK({required String uid}) async {
    return _channel.invokeMethod("init", <String, dynamic>{
      'uid': uid,
    });
  }

  Future<void> show(String placement) {
    return _channel.invokeMethod("show", <String, dynamic>{
      'placement': placement,
    });
  }

  Future<void> deduct(int amount) {
    return _channel.invokeMethod("deduct", <String, dynamic>{
      'amount': amount,
    });
  }

  Future _platformCallHandler(MethodCall call) async {
    switch (call.method) {
      case "userBalanceChanged":
        _userBalanceChanged(call.arguments);
        log("userBalanceChanged");
        break;
      case "userAvailableBalance":
        _userAvailableBalance(call.arguments);
        log("userAvailableBalance");
        break;
      case "userPendingBalance":
        _userPendingBalance(call.arguments);
        log("userPendingBalance");
        break;
      case "initializationFailed":
        _initializationFailed(call.arguments);
        log("initializationFailed");
        break;
      case "deductUserBalance":
        _deductUserBalance(call.arguments);
        break;
      default:
        log('Unknown method ${call.method} ');
    }
  }

  void setUserBalanceChanged(UserBalanceChanged userBalanceChanged) => _userBalanceChanged = userBalanceChanged;

  void setUserAvailableBalance(UserAvailableBalance userAvailableBalance) => _userAvailableBalance = userAvailableBalance;

  void setUserPendingBalance(UserAvailableBalance userPendingBalance) => _userPendingBalance = userPendingBalance;

  void setInitilizationFailedListener(InitializationFailed initializationFailed) => _initializationFailed = initializationFailed;
  void setDeductUserBalanceListener(DeductUserBalance deductUserBalance) => _deductUserBalance = deductUserBalance;
}

class CsAyetstudiosIos {
  final MethodChannel _channel = const MethodChannel('cs_ayetstudios_ios');
  static final CsAyetstudiosIos _instance = CsAyetstudiosIos._internal();
  factory CsAyetstudiosIos() => _instance;
  CsAyetstudiosIos._internal() {
    _channel.setMethodCallHandler(_platformCallHandler);
  }

  Future<void> initSDK({required String uid, appKey}) async {
    return _channel.invokeMethod("init", <String, dynamic>{
      'appKey': appKey,
      'uid': uid,
    });
  }

  late UserBalanceChanged _userBalanceChanged;
  late UserAvailableBalance _userAvailableBalance;
  late UserPendingBalance _userPendingBalance;

  void sdkLogEnable() async {
    return _channel.invokeMethod("sdkLogEnable");
  }

  void userAvailableBalance() async {
    return _channel.invokeMethod("userAvailableBalance");
  }

  void userPendingBalance() async {
    return _channel.invokeMethod("userPendingBalance");
  }

  Future<void> show(String placement) {
    return _channel.invokeMethod("show", <String, dynamic>{
      'placement': placement,
    });
  }

  Future _platformCallHandler(MethodCall call) async {
    switch (call.method) {
      case "userBalanceChanged":
        _userBalanceChanged(call.arguments);
        print("userBalanceChanged");
        break;
      case "userAvailableBalance":
        _userAvailableBalance(call.arguments);
        print("userAvailableBalance");
        break;
      case "userPendingBalance":
        _userPendingBalance(call.arguments);
        print("userPendingBalance");
        break;
      default:
        print('Unknown method ${call.method} ');
    }
  }

  void setUserBalanceChanged(UserBalanceChanged userBalanceChanged) => _userBalanceChanged = userBalanceChanged;

  void setUserAvailableBalance(UserAvailableBalance userAvailableBalance) => _userAvailableBalance = userAvailableBalance;

  void setUserPendingBalance(UserAvailableBalance userPendingBalance) => _userPendingBalance = userPendingBalance;
}
